@@ -160,7 +160,7 @@ class Notification(Message):
         )
 
     @classmethod
-    def unpack_message(cls, data, negotiated=None):
+    def unpack_message(cls, data, direction=None, negotiated=None):
         return cls(ordinal(data[0]), ordinal(data[1]), data[2:])
 
 
