@@ -25,12 +25,12 @@ class ExtendedCommunities(Communities):
     ID = Attribute.CODE.EXTENDED_COMMUNITY
 
     @staticmethod
-    def unpack(data, negotiated):
+    def unpack(data, direction, negotiated):
         communities = ExtendedCommunities()
         while data:
             if data and len(data) < 8:
                 raise Notify(3, 1, 'could not decode extended community %s' % str([hex(ordinal(_)) for _ in data]))
-            communities.add(ExtendedCommunity.unpack(data[:8], negotiated))
+            communities.add(ExtendedCommunity.unpack(data[:8], direction, negotiated))
             data = data[8:]
         return communities
 
@@ -44,11 +44,11 @@ class ExtendedCommunitiesIPv6(Communities):
     ID = Attribute.CODE.IPV6_EXTENDED_COMMUNITY
 
     @staticmethod
-    def unpack(data, negotiated):
+    def unpack(data, direction, negotiated):
         communities = ExtendedCommunitiesIPv6()
         while data:
             if data and len(data) < 20:
                 raise Notify(3, 1, 'could not decode ipv6 extended community %s' % str([hex(ordinal(_)) for _ in data]))
-            communities.add(ExtendedCommunityIPv6.unpack(data[:20], negotiated))
+            communities.add(ExtendedCommunityIPv6.unpack(data[:20], direction, negotiated))
             data = data[20:]
         return communities
