@@ -91,7 +91,7 @@ class OutgoingRIB(Cache):
                 if family not in self._enhanced_refresh_start:
                     self._enhanced_refresh_start.append(family)
 
-        changes = list(self.cached_changes(requested_families))
+        changes = list(self.cached_changes(requested_families, [OUT.ANNOUNCE,OUT.WITHDRAW]))
         for change in changes:
             self.del_from_rib(change)
 
@@ -100,9 +100,12 @@ class OutgoingRIB(Cache):
             yield change
 
     def replace(self, previous, changes):
+        nlri = [c.nlri for c in changes]
+
         for change in previous:
-            change.nlri.action = OUT.WITHDRAW
-            self.add_to_rib(change, True)
+            if change.nlri not in nlri:
+                change.nlri.action = OUT.WITHDRAW
+                self.add_to_rib(change, True)
 
         for change in changes:
             self.add_to_rib(change, True)
