@@ -30,6 +30,7 @@ class DocoptExit(SystemExit):
 
 
 class Pattern(object):
+
     def __eq__(self, other):
         return repr(self) == repr(other)
 
@@ -46,18 +47,18 @@ class Pattern(object):
         if not hasattr(self, 'children'):
             return self
         uniq = list(set(self.flat())) if uniq is None else uniq
-        for i, c in enumerate(self.children):
-            if not hasattr(c, 'children'):
-                assert c in uniq
-                self.children[i] = uniq[uniq.index(c)]
+        for i, child in enumerate(self.children):
+            if not hasattr(child, 'children'):
+                assert child in uniq
+                self.children[i] = uniq[uniq.index(child)]
             else:
-                c.fix_identities(uniq)
+                child.fix_identities(uniq)
 
     def fix_repeating_arguments(self):
         """Fix elements that should accumulate/increment values."""
-        either = [list(c.children) for c in self.either.children]
+        either = [list(child.children) for child in transform(self).children]
         for case in either:
-            for e in [c for c in case if case.count(c) > 1]:
+            for e in [child for child in case if case.count(child) > 1]:
                 if type(e) is Argument or type(e) is Option and e.argcount:
                     if e.value is None:
                         e.value = []
@@ -67,46 +68,40 @@ class Pattern(object):
                     e.value = 0
         return self
 
-    @property
-    def either(self):
-        """Transform pattern into an equivalent, with only top-level Either."""
-        # Currently the pattern will not be equivalent, but more "narrow",
-        # although good enough to reason about list arguments.
-        ret = []
-        groups = [[self]]
-        while groups:
-            children = groups.pop(0)
-            types = [type(c) for c in children]
-            if Either in types:
-                either = [c for c in children if type(c) is Either][0]
-                children.pop(children.index(either))
-                for c in either.children:
+
+def transform(pattern):
+    """Expand pattern into an (almost) equivalent one, but with single Either.
+
+    Example: ((-a | -b) (-c | -d)) => (-a -c | -a -d | -b -c | -b -d)
+    Quirks: [-a] => (-a), (-a...) => (-a -a)
+
+    """
+    result = []
+    groups = [[pattern]]
+    while groups:
+        children = groups.pop(0)
+        parents = [Required, Optional, OptionsShortcut, Either, OneOrMore]
+        if any(t in map(type, children) for t in parents):
+            child = [c for c in children if type(c) in parents][0]
+            children.remove(child)
+            if type(child) is Either:
+                for c in child.children:
                     groups.append([c] + children)
-            elif Required in types:
-                required = [c for c in children if type(c) is Required][0]
-                children.pop(children.index(required))
-                groups.append(list(required.children) + children)
-            elif Optional in types:
-                optional = [c for c in children if type(c) is Optional][0]
-                children.pop(children.index(optional))
-                groups.append(list(optional.children) + children)
-            elif AnyOptions in types:
-                optional = [c for c in children if type(c) is AnyOptions][0]
-                children.pop(children.index(optional))
-                groups.append(list(optional.children) + children)
-            elif OneOrMore in types:
-                oneormore = [c for c in children if type(c) is OneOrMore][0]
-                children.pop(children.index(oneormore))
-                groups.append(list(oneormore.children) * 2 + children)
+            elif type(child) is OneOrMore:
+                groups.append(child.children * 2 + children)
             else:
-                ret.append(children)
-        return Either(*[Required(*e) for e in ret])
+                groups.append(child.children + children)
+        else:
+            result.append(children)
+    return Either(*[Required(*e) for e in result])
 
 
-class ChildPattern(Pattern):
+class LeafPattern(Pattern):
+
+    """Leaf/terminal node of a pattern tree."""
+
     def __init__(self, name, value=None):
-        self.name = name
-        self.value = value
+        self.name, self.value = name, value
 
     def __repr__(self):
         return '%s(%r, %r)' % (self.__class__.__name__, self.name, self.value)
@@ -119,13 +114,14 @@ class ChildPattern(Pattern):
         pos, match = self.single_match(left)
         if match is None:
             return False, left, collected
-        left_ = left[:pos] + left[pos + 1 :]
+        left_ = left[:pos] + left[pos + 1:]
         same_name = [a for a in collected if a.name == self.name]
         if type(self.value) in (int, list):
             if type(self.value) is int:
                 increment = 1
             else:
-                increment = [match.value] if type(match.value) is str else match.value
+                increment = ([match.value] if type(match.value) is str
+                             else match.value)
             if not same_name:
                 match.value = increment
                 return True, left_, collected + [match]
@@ -134,57 +130,62 @@ class ChildPattern(Pattern):
         return True, left_, collected + [match]
 
 
-class ParentPattern(Pattern):
+class BranchPattern(Pattern):
+
+    """Branch/inner node of a pattern tree."""
+
     def __init__(self, *children):
         self.children = list(children)
 
     def __repr__(self):
-        return '%s(%s)' % (self.__class__.__name__, ', '.join(repr(a) for a in self.children))
+        return '%s(%s)' % (self.__class__.__name__,
+                           ', '.join(repr(a) for a in self.children))
 
     def flat(self, *types):
         if type(self) in types:
             return [self]
-        return sum([c.flat(*types) for c in self.children], [])
+        return sum([child.flat(*types) for child in self.children], [])
 
 
-class Argument(ChildPattern):
+class Argument(LeafPattern):
+
     def single_match(self, left):
-        for n, p in enumerate(left):
-            if type(p) is Argument:
-                return n, Argument(self.name, p.value)
+        for n, pattern in enumerate(left):
+            if type(pattern) is Argument:
+                return n, Argument(self.name, pattern.value)
         return None, None
 
     @classmethod
-    def parse(cls, source):
-        name = re.findall('(<\S*?>)', source)[0]
-        value = re.findall('\[default: (.*)\]', source, flags=re.I)
-        return cls(name, value[0] if value else None)
+    def parse(class_, source):
+        name = re.findall(r'(<\S*?>)', source)[0]
+        value = re.findall(r'\[default: (.*)\]', source, flags=re.I)
+        return class_(name, value[0] if value else None)
 
 
 class Command(Argument):
+
     def __init__(self, name, value=False):
-        self.name = name
-        self.value = value
+        self.name, self.value = name, value
 
     def single_match(self, left):
-        for n, p in enumerate(left):
-            if type(p) is Argument:
-                if p.value == self.name:
+        for n, pattern in enumerate(left):
+            if type(pattern) is Argument:
+                if pattern.value == self.name:
                     return n, Command(self.name, True)
                 else:
                     break
         return None, None
 
 
-class Option(ChildPattern):
+class Option(LeafPattern):
+
     def __init__(self, short=None, long=None, argcount=0, value=False):
         assert argcount in (0, 1)
-        self.short, self.long = short, long
-        self.argcount, self.value = argcount, value
+        self.short, self.long, self.argcount = short, long, argcount
         self.value = None if value is False and argcount else value
 
     @classmethod
-    def parse(cls, option_description):
+    def parse(class_, option_description):
         short, long, argcount, value = None, None, 0, False
         options, _, description = option_description.strip().partition('  ')
         options = options.replace(',', ' ').replace('=', ' ')
@@ -196,14 +197,14 @@ class Option(ChildPattern):
             else:
                 argcount = 1
         if argcount:
-            matched = re.findall('\[default: (.*)\]', description, flags=re.I)
+            matched = re.findall(r'\[default: (.*)\]', description, flags=re.I)
             value = matched[0] if matched else None
-        return cls(short, long, argcount, value)
+        return class_(short, long, argcount, value)
 
     def single_match(self, left):
-        for n, p in enumerate(left):
-            if self.name == p.name:
-                return n, p
+        for n, pattern in enumerate(left):
+            if self.name == pattern.name:
+                return n, pattern
         return None, None
 
     @property
@@ -211,35 +212,39 @@ class Option(ChildPattern):
         return self.long or self.short
 
     def __repr__(self):
-        return 'Option(%r, %r, %r, %r)' % (self.short, self.long, self.argcount, self.value)
+        return 'Option(%r, %r, %r, %r)' % (self.short, self.long,
+                                           self.argcount, self.value)
 
 
-class Required(ParentPattern):
+class Required(BranchPattern):
+
     def match(self, left, collected=None):
         collected = [] if collected is None else collected
         l = left
         c = collected
-        for p in self.children:
-            matched, l, c = p.match(l, c)
+        for pattern in self.children:
+            matched, l, c = pattern.match(l, c)
             if not matched:
                 return False, left, collected
         return True, l, c
 
 
-class Optional(ParentPattern):
+class Optional(BranchPattern):
+
     def match(self, left, collected=None):
         collected = [] if collected is None else collected
-        for p in self.children:
-            m, left, collected = p.match(left, collected)
+        for pattern in self.children:
+            m, left, collected = pattern.match(left, collected)
         return True, left, collected
 
 
-class AnyOptions(Optional):
+class OptionsShortcut(Optional):
 
     """Marker/placeholder for [options] shortcut."""
 
 
-class OneOrMore(ParentPattern):
+class OneOrMore(BranchPattern):
+
     def match(self, left, collected=None):
         assert len(self.children) == 1
         collected = [] if collected is None else collected
@@ -260,12 +265,13 @@ class OneOrMore(ParentPattern):
         return False, left, collected
 
 
-class Either(ParentPattern):
+class Either(BranchPattern):
+
     def match(self, left, collected=None):
         collected = [] if collected is None else collected
         outcomes = []
-        for p in self.children:
-            matched, _, _ = outcome = p.match(left, collected)
+        for pattern in self.children:
+            matched, _, _ = outcome = pattern.match(left, collected)
             if matched:
                 outcomes.append(outcome)
         if outcomes:
@@ -273,11 +279,18 @@ class Either(ParentPattern):
         return False, left, collected
 
 
-class TokenStream(list):
-    def __init__(self, source, error):
+class Tokens(list):
+
+    def __init__(self, source, error=DocoptExit):
         self += source.split() if hasattr(source, 'split') else source
         self.error = error
 
+    @staticmethod
+    def from_pattern(source):
+        source = re.sub(r'([\[\]\(\)\|]|\.\.\.)', r' \1 ', source)
+        source = [s for s in re.split(r'\s+|(\S*<.*?>)', source) if s]
+        return Tokens(source, error=DocoptLanguageError)
+
     def move(self):
         return self.pop(0) if len(self) else None
 
@@ -294,7 +307,8 @@ def parse_long(tokens, options):
     if tokens.error is DocoptExit and similar == []:  # if no exact match
         similar = [o for o in options if o.long and o.long.startswith(long)]
     if len(similar) > 1:  # might be simply specified ambiguously 2+ times?
-        raise tokens.error('%s is not a unique prefix: %s?' % (long, ', '.join(o.long for o in similar)))
+        raise tokens.error('%s is not a unique prefix: %s?' %
+                           (long, ', '.join(o.long for o in similar)))
     elif len(similar) < 1:
         argcount = 1 if eq == '=' else 0
         o = Option(None, long, argcount)
@@ -302,13 +316,14 @@ def parse_long(tokens, options):
         if tokens.error is DocoptExit:
             o = Option(None, long, argcount, value if argcount else True)
     else:
-        o = Option(similar[0].short, similar[0].long, similar[0].argcount, similar[0].value)
+        o = Option(similar[0].short, similar[0].long,
+                   similar[0].argcount, similar[0].value)
         if o.argcount == 0:
             if value is not None:
                 raise tokens.error('%s must not have an argument' % o.long)
         else:
             if value is None:
-                if tokens.current() is None:
+                if tokens.current() in [None, '--']:
                     raise tokens.error('%s requires argument' % o.long)
                 value = tokens.move()
         if tokens.error is DocoptExit:
@@ -326,18 +341,20 @@ def parse_shorts(tokens, options):
         short, left = '-' + left[0], left[1:]
         similar = [o for o in options if o.short == short]
         if len(similar) > 1:
-            raise tokens.error('%s is specified ambiguously %d times' % (short, len(similar)))
+            raise tokens.error('%s is specified ambiguously %d times' %
+                               (short, len(similar)))
         elif len(similar) < 1:
             o = Option(short, None, 0)
             options.append(o)
             if tokens.error is DocoptExit:
                 o = Option(short, None, 0, True)
         else:  # why copying is necessary here?
-            o = Option(short, similar[0].long, similar[0].argcount, similar[0].value)
+            o = Option(short, similar[0].long,
+                       similar[0].argcount, similar[0].value)
             value = None
             if o.argcount != 0:
                 if left == '':
-                    if tokens.current() is None:
+                    if tokens.current() in [None, '--']:
                         raise tokens.error('%s requires argument' % short)
                     value = tokens.move()
                 else:
@@ -350,7 +367,7 @@ def parse_shorts(tokens, options):
 
 
 def parse_pattern(source, options):
-    tokens = TokenStream(re.sub(r'([\[\]\(\)\|]|\.\.\.)', r' \1 ', source), DocoptLanguageError)
+    tokens = Tokens.from_pattern(source)
     result = parse_expr(tokens, options)
     if tokens.current() is not None:
         raise tokens.error('unexpected ending: %r' % ' '.join(tokens))
@@ -397,7 +414,7 @@ def parse_atom(tokens, options):
         return [result]
     elif token == 'options':
         tokens.move()
-        return [AnyOptions()]
+        return [OptionsShortcut()]
     elif token.startswith('--') and token != '--':
         return parse_long(tokens, options)
     elif token.startswith('-') and token not in ('-', '--'):
@@ -433,27 +450,26 @@ def parse_argv(tokens, options, options_first=False):
 
 
 def parse_defaults(doc):
-    # in python < 2.7 you can't pass flags=re.MULTILINE
-    split = re.split('\n *(<\S+?>|-\S+?)', doc)[1:]
-    split = [s1 + s2 for s1, s2 in zip(split[::2], split[1::2])]
-    options = [Option.parse(s) for s in split if s.startswith('-')]
-    # arguments = [Argument.parse(s) for s in split if s.startswith('<')]
-    # return options, arguments
-    return options
+    defaults = []
+    for s in parse_section('options:', doc):
+        # FIXME corner case "bla: options: --foo"
+        _, _, s = s.partition(':')  # get rid of "options:"
+        split = re.split(r'\n[ \t]*(-\S+?)', '\n' + s)[1:]
+        split = [s1 + s2 for s1, s2 in zip(split[::2], split[1::2])]
+        options = [Option.parse(s) for s in split if s.startswith('-')]
+        defaults += options
+    return defaults
 
 
-def printable_usage(doc):
-    # in python < 2.7 you can't pass flags=re.IGNORECASE
-    usage_split = re.split(r'([Uu][Ss][Aa][Gg][Ee]:)', doc)
-    if len(usage_split) < 3:
-        raise DocoptLanguageError('"usage:" (case-insensitive) not found.')
-    if len(usage_split) > 3:
-        raise DocoptLanguageError('More than one "usage:" (case-insensitive).')
-    return re.split(r'\n\s*\n', ''.join(usage_split[1:]))[0].strip()
+def parse_section(name, source):
+    pattern = re.compile('^([^\n]*' + name + '[^\n]*\n?(?:[ \t].*?(?:\n|$))*)',
+                         re.IGNORECASE | re.MULTILINE)
+    return [s.strip() for s in pattern.findall(source)]
 
 
-def formal_usage(printable_usage):
-    pu = printable_usage.split()[1:]  # split and drop "usage:"
+def formal_usage(section):
+    _, _, section = section.partition(':')  # drop "usage:"
+    pu = section.split()
     return '( ' + ' '.join(') | (' if s == pu[0] else s for s in pu[1:]) + ' )'
 
 
@@ -493,7 +509,7 @@ def docopt(doc, argv=None, help=True, version=None, options_first=False):
         If passed, the object will be printed if --version is in
         `argv`.
     options_first : bool (default: False)
-        Set to True to require options preceed positional arguments,
+        Set to True to require options precede positional arguments,
         i.e. to forbid options and positional arguments intermix.
 
     Returns
@@ -507,15 +523,15 @@ def docopt(doc, argv=None, help=True, version=None, options_first=False):
     -------
     >>> from docopt import docopt
     >>> doc = '''
-    Usage:
-        my_program tcp <host> <port> [--timeout=<seconds>]
-        my_program serial <port> [--baud=<n>] [--timeout=<seconds>]
-        my_program (-h | --help | --version)
-
-    Options:
-        -h, --help  Show this screen and exit.
-        --baud=<n>  Baudrate [default: 9600]
-    '''
+    ... Usage:
+    ...     my_program tcp <host> <port> [--timeout=<seconds>]
+    ...     my_program serial <port> [--baud=<n>] [--timeout=<seconds>]
+    ...     my_program (-h | --help | --version)
+    ...
+    ... Options:
+    ...     -h, --help  Show this screen and exit.
+    ...     --baud=<n>  Baudrate [default: 9600]
+    ... '''
     >>> argv = ['tcp', '127.0.0.1', '80', '--timeout', '30']
     >>> docopt(doc, argv)
     {'--baud': '9600',
@@ -534,23 +550,29 @@ def docopt(doc, argv=None, help=True, version=None, options_first=False):
       at https://github.com/docopt/docopt#readme
 
     """
-    if argv is None:
-        argv = sys.argv[1:]
-    DocoptExit.usage = printable_usage(doc)
+    argv = sys.argv[1:] if argv is None else argv
+
+    usage_sections = parse_section('usage:', doc)
+    if len(usage_sections) == 0:
+        raise DocoptLanguageError('"usage:" (case-insensitive) not found.')
+    if len(usage_sections) > 1:
+        raise DocoptLanguageError('More than one "usage:" (case-insensitive).')
+    DocoptExit.usage = usage_sections[0]
+
     options = parse_defaults(doc)
     pattern = parse_pattern(formal_usage(DocoptExit.usage), options)
     # [default] syntax for argument is disabled
-    # for a in pattern.flat(Argument):
+    #for a in pattern.flat(Argument):
     #    same_name = [d for d in arguments if d.name == a.name]
     #    if same_name:
     #        a.value = same_name[0].value
-    argv = parse_argv(TokenStream(argv, DocoptExit), list(options), options_first)
+    argv = parse_argv(Tokens(argv), list(options), options_first)
     pattern_options = set(pattern.flat(Option))
-    for ao in pattern.flat(AnyOptions):
+    for options_shortcut in pattern.flat(OptionsShortcut):
         doc_options = parse_defaults(doc)
-        ao.children = list(set(doc_options) - pattern_options)
-        # if any_options:
-        #    ao.children += [Option(o.short, o.long, o.argcount)
+        options_shortcut.children = list(set(doc_options) - pattern_options)
+        #if any_options:
+        #    options_shortcut.children += [Option(o.short, o.long, o.argcount)
         #                    for o in argv if type(o) is Option]
     extras(help, version, argv, doc)
     matched, left, collected = pattern.fix().match(argv)
